@@ -6,16 +6,6 @@
 # This software is copyright (c) 2014 by Gideon Knocke.
 # Copyright (c) 2016 Gideon Knocke, Timothe Litt
 #
-# This is free software; you can redistribute it and/or modify it under
-# the same terms as the Perl 5 programming language system itself.
-#
-# Terms of the Perl programming language system itself
-#
-# a) the GNU General Public License as published by the Free
-#   Software Foundation; either version 1, or (at your option) any
-#   later version, or
-# b) the "Artistic License"
-#
 # See LICENSE for details.
 #
 
@@ -32,7 +22,7 @@ use Encode ();
 use MIME::Base64;
 use Scalar::Util ();
 
-our $VERSION = 1.5;
+our $VERSION = 1.6;
 
 my $apiVersion = undef;  # 0 for compatibility.  1 for prefered
 my $error;
@@ -81,6 +71,11 @@ my %oids = (
     '2.5.29.19'                     => [ 'basicConstraints', 'Basic Constraints' ],
     '1.2.840.10040.4.1'             => [ 'dsa', 'DSA' ],
     '1.2.840.10040.4.3'             => [ 'dsaWithSha1', 'DSA with SHA1' ],
+    '1.2.840.10045.2.1'             => 'ecPublicKey',
+    '1.2.840.10045.4.3.1'           => 'ecdsa-with-SHA224',
+    '1.2.840.10045.4.3.2'           => 'ecdsa-with-SHA256',
+    '1.2.840.10045.4.3.3'           => 'ecdsa-with-SHA384',
+    '1.2.840.10045.4.3.4'           => 'ecdsa-with-SHA512',
     '0.9.2342.19200300.100.1.25'    => 'domainComponent',
     '0.9.2342.19200300.100.1.1'     => 'userID',
     '2.5.4.7'                       => 'localityName',
@@ -196,21 +191,21 @@ my %name2oid;
 
 # For generating documentation, not part of API
 
+sub _cmpOID {
+    my @a = split( /\./, $a );
+    my @b = split( /\./, $b );
+
+    while( @a && @b ) {
+        my $c = shift @a <=> shift @b;
+        return $c if( $c );
+    }
+    return @a <=> @b;
+}
+
 sub __listOIDs {
     my $class = shift;
     my ( $hash ) = @_;
 
-    sub _cmpOID {
-	my @a = split( /\./, $a );
-	my @b = split( /\./, $b );
-
-	while( @a && @b ) {
-	    my $c = shift @a <=> shift @b;
-	    return $c if( $c );
-	}
-	return @a <=> @b;
-    }
-
     my @max = (0) x 3;
     foreach my $oid ( keys %$hash ) {
 	my $len;
@@ -271,13 +266,11 @@ sub name2oid {
     my $class = shift;
     my( $oid ) = @_;
 
-    return undef unless( defined $oid && $apiVersion > 0 );
+    return unless( defined $oid && $apiVersion > 0 );
 
     return $name2oid{$oid};
 }
 
-# Currently undocumented
-
 sub oid2name {
     my $class = shift;
     my( $oid ) = @_;
@@ -337,7 +330,7 @@ sub new {
     }; if( $@ ) {
 	$error = $@;
 	croak( $@ ) if( $apiVersion == 0 );
-	return undef;
+	return;
     }
 
     return $self;
@@ -445,7 +438,7 @@ sub _new {
 
     AlgorithmIdentifier ::= SEQUENCE {
       algorithm  OBJECT IDENTIFIER,
-      parameters Algorithms}
+      parameters Algorithms OPTIONAL}
 
     SubjectPublicKeyInfo ::= SEQUENCE {
       algorithm        AlgorithmIdentifier,
@@ -940,7 +933,7 @@ sub _convert_rdn {
 		my @names = (exists $variantNames{$oid})? @{$variantNames{$oid}} : ( $name );
 		foreach my $name ( @names ) {
 		    unless( $self->can( $name ) ) {
-			no strict 'refs';
+			no strict 'refs'; ## no critic
 			*$name =  sub {
 			    my $self = shift;
 			    return @{ $self->{certificationRequestInfo}{subject}{$name} } if( wantarray );
@@ -985,7 +978,7 @@ sub csrRequest {
 
 foreach my $component (qw/commonName organizationalUnitName organizationName
                           emailAddress stateOrProvinceName countryName domainComponent/ ) {
-    no strict 'refs';
+    no strict 'refs'; ## no critic
 
     unless( defined &$component ) {
 	*$component = sub {
@@ -1023,14 +1016,14 @@ sub subjectAltName {
     my $san = $self->extensionValue( 'subjectAltName' );
     unless( defined $san ) {
 	return () if( wantarray );
-	return undef;
+	return undef;  ## no critic
     }
 
     if( !defined $type ) {
 	if( wantarray ) {
 	    my %comps;
 	    $comps{$_} = 1 foreach (map { keys %$_ } @$san);
-	    return sort keys %comps;
+	    return sort keys %comps; ## no critic
 	}
 	my @string;
 	foreach my $comp (@$san) {
@@ -1078,7 +1071,7 @@ sub _attributes {
     my $self = shift;
 
     my $attributes = $self->{certificationRequestInfo}{attributes};
-    return undef unless( defined $attributes );
+    return unless( defined $attributes );
 
     return { map { $_->{type} => $_->{values} } @$attributes };
 }
@@ -1099,7 +1092,7 @@ sub attributes {
     my $attributes = $self->_attributes;
     unless( defined $attributes ) {
 	return () if( wantarray );
-	return undef;
+	return undef;  ## no critic
     }
 
     unless( defined $name ) {
@@ -1110,7 +1103,7 @@ sub attributes {
 
     if( $name eq 'extensionRequest' ) { # Meaningless, and extensions/extensionValue handle
 	return () if( wantarray );
-	return undef;
+	return undef; ## no critic
     }
 
     # There can only be one matching the name.
@@ -1120,7 +1113,7 @@ sub attributes {
     my @attrs = grep { $_ eq $name } keys %$attributes;
     unless( @attrs ) {
 	return () if( wantarray );
-	return undef;
+	return undef; ## no critic
     }
 
     my @values;
@@ -1223,7 +1216,7 @@ sub extensionValue {
 
     my $attributes = $self->_attributes;
     my $value;
-    return undef unless( defined $attributes && exists $attributes->{extensionRequest} );
+    return unless( defined $attributes && exists $attributes->{extensionRequest} );
 
     $extensionName = $self->_oid2name( $extensionName );
 
@@ -1258,7 +1251,7 @@ sub extensionPresent {
     my( $extensionName ) = @_;
 
     my $attributes = $self->_attributes;
-    return undef unless( defined $attributes && exists $attributes->{extensionRequest} );
+    return unless( defined $attributes && exists $attributes->{extensionRequest} );
 
     $extensionName = $self->_oid2name( $extensionName );
 
@@ -1270,7 +1263,7 @@ sub extensionPresent {
 	    return 1;
         }
     }
-    return undef;
+    return;
 }
 
 sub _wrap {
@@ -1348,7 +1341,7 @@ Crypt::PKCS10 - parse PKCS #10 certificate requests
 
 =head1 VERSION
 
-version 1.5
+version 1.6
 
 =head1 SYNOPSIS
 
@@ -1374,18 +1367,17 @@ Bit Strings like signatures will be returned in their hexadecimal representation
 
 The access methods return the value corresponding to their name.  If called in scalar context, they return the first value (or an empty string).  If called in array context, they return all values.
 
-=begin readme text
+=begin :readme
 
-This file is automatically generated by pod2readme from PKCS10.pm and Changes.
+  This file is automatically generated by pod2readme from PKCS10.pm and Changes.
 
-
-=end readme
+=end :readme
 
 =head1 NAME
 
 Crypt::PKCS10 - parse PKCS #10 certificate requests
 
-=begin readme pod
+=begin :readme
 
 =head1 RELEASE NOTES
 
@@ -1404,20 +1396,21 @@ new will accept an open file handle in addition to a request.
 
 Users are encouraged to migrate to the version 1 API.  It is much easier to use,
 and does not require the application to navigate internal data structures.
-=head1 INSTALLATION
-    To install this module type the following:
 
-       perl Makefile.PL
-       make
-       make test
-       make install
+=head1 INSTALLATION
+
+To install this module type the following:
+
+    perl Makefile.PL
+    make
+    make test
+    make install
 
 =head1 REQUIRES
 
 Convert::ASN1
 
-
-=end readme
+=end :readme
 
 =head1 METHODS
 
@@ -1655,12 +1648,13 @@ See the I<Table of known OID names> below for a list of names.
 
 =for readme continue
 
-=begin readme pod
+=begin :readme
 
 See the module documentation for a list of known OID names.
 
+It is too long to include here.
 
-=end readme
+=end :readme
 
 =head2 extensions
 
@@ -1697,12 +1691,13 @@ See the I<Table of known OID names> below for a list of names.
 
 =for readme continue
 
-=begin readme pod
+=begin :readme
 
 See the module documentation for a list of known OID names.
 
+It is too long to include here.
 
-=end readme
+=end :readme
 
 =head2 extensionPresent( $name )
 
@@ -1720,12 +1715,13 @@ See the I<Table of known OID names> below for a list of names.
 
 =for readme continue
 
-=begin readme pod
+=begin :readme
 
 See the module documentation for a list of known OID names.
 
+It is too long to include here.
 
-=end readme
+=end :readme
 
 =head2 registerOID( )
 
@@ -1783,6 +1779,11 @@ B<registerOID>.
  0.9.2342.19200300.100.1.25 domainComponent
  1.2.840.10040.4.1          dsa                        (DSA)
  1.2.840.10040.4.3          dsaWithSha1                (DSA with SHA1)
+ 1.2.840.10045.2.1          ecPublicKey
+ 1.2.840.10045.4.3.1        ecdsa-with-SHA224
+ 1.2.840.10045.4.3.2        ecdsa-with-SHA256
+ 1.2.840.10045.4.3.3        ecdsa-with-SHA384
+ 1.2.840.10045.4.3.4        ecdsa-with-SHA512
  1.2.840.113549.1.1.1       rsaEncryption              (RSA encryption)
  1.2.840.113549.1.1.2       md2WithRSAEncryption       (MD2 with RSA encryption)
  1.2.840.113549.1.1.3       md4WithRSAEncryption
@@ -1885,7 +1886,7 @@ B<registerOID>.
 
 =for readme continue
 
-=begin readme pod
+=begin :readme
 
 =head1 CHANGES
 
@@ -1893,8 +1894,7 @@ B<registerOID>.
 
 For a more detailed list of changes, see F<Commitlog> in the distribution.
 
-
-=end readme
+=end :readme
 
 =head1 EXAMPLES
 
@@ -1911,32 +1911,18 @@ tested does not imply that they are stable, or that they will be present in any
 The test data was selected to exercise the API; the CSR contents are not representative of
 realistic certificate requests.
 
-=head1 AUTHORS
+=head1 ACKNOWLEDGEMENTS
 
-Gideon Knocke
+Martin Bartosch contributed the OIDs and tests for EC support.
 
-Timothe Litt made most of the changes for V1.4
+Timothe Litt made most of the changes for V1.4+
 
 C<Crypt::PKCS10> is based on the generic ASN.1 module by Graham Barr and on the
  x509decode example by Norbert Klasen. It is also based upon the
 works of Duncan Segrest's C<Crypt-X509-CRL> module.
 
-=head1 COPYRIGHT
+=for readme start
 
-This software is copyright (c) 2014 by Gideon Knocke.
-
-Changes in V1.4 are copyright (C) 2016, Timothe Litt
-
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
-
-Terms of the Perl programming language system itself
-
-a) the GNU General Public License as published by the Free
-   Software Foundation; either version 1, or (at your option) any
-   later version, or
-
-b) the "Artistic License"
 
 =head1 AUTHORS
 
@@ -1954,9 +1940,19 @@ Timothe Litt <tlhackque@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Gideon Knocke.
+This software is copyright (c) 2014, 2016 by Gideon Knocke, Timothe Litt.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
+=head1 BUG REPORTS
+
+Please report any bugs or feature requests on the bugtracker website
+https://rt.cpan.org/Public/Dist/Display.html?Name=Crypt-PKCS10 or by email
+to bug-crypt-pkcs10@rt.cpan.org.
+
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
+
 =cut
